#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /usr/share/elma-tests/bin/elma_diag_all_after_update.sh
#

# async test for gzip file writing and elasticsearch functionality

SCRIPT_DESCRIPTION="rsyslogd performance and compression tests after system update"
SCRIPT_AUTHOR="Joerg Heinemann"

elasticsearchdiagindex="diag"
elasticsearchdiagindexnocompression="diag-no-compression"

# ######### Load ELMA configuration parameter, global functions and script header #########
#
source /etc/elma/elma.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $ELMA_LIB_DIR/elma-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
script_header ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

export testsdir="$ELMA_TESTS_LIB_DIR" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
cd $testsdir ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

echo --- file output with gzip compression
echo
echo  ===========================================================================================
echo

source $testsdir/diag.sh init ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh startup $RSYSLOG_CONF_DIR/elma_diag_imptcp_gzipwr.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# send 10.000 messages of 200 bytes plus header, 10 times
# Only 9 option parameter will be accepted!

source $testsdir/diag.sh -d200 -m1000 -R100 -S0 -c1 -Ttcp -p13514 -t127.0.0.1 tcpflood ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh wait-tcp-receiver-to-settle 2	# due to large messages, we need this time for the tcp receiver to settle...
source $testsdir/diag.sh shutdown-when-empty		# shut down rsyslogd when done processing messages
source $testsdir/diag.sh wait-shutdown			# and wait for it to terminate

TOTALMESSAGES=$(( $TCPFLOOD_TOTALMESSAGES - 1 ))
source $testsdir/diag.sh gzip-seq-check 0 $TOTALMESSAGES -E ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh exit $RETVAL "$testname  (file output with gzip compression)" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

echo  --- elasticsearch output
echo
echo  ===========================================================================================
echo

source $testsdir/diag.sh init ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh delete-elasticsearch-index $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh startup $RSYSLOG_CONF_DIR/elma_diag_imptcp_elasticsearch.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# send 10.000 messages of 200 bytes plus header, 10 times
# Only 9 option parameter will be accepted!

source $testsdir/diag.sh -d200 -m1000 -R100 -S0 -c1 -Ttcp -p13514 -t127.0.0.1 tcpflood ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh elasticsearch-index-check $elasticsearchdiagindex $TCPFLOOD_TOTALMESSAGES ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh elasticsearch-index-statistics $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh elasticsearch-index-status $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh elasticsearch-index-settings $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh elasticsearch-index-mappings $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

#source $testsdir/diag.sh wait-tcp-receiver-to-settle 2	# due to large messages, we need this time for the tcp receiver to settle...
source $testsdir/diag.sh shutdown-when-empty		# shut down rsyslogd when done processing messages
source $testsdir/diag.sh wait-shutdown			# and wait for it to terminate

#source $testsdir/diag.sh delete-elasticsearch-index $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh exit $RETVAL "$testname  (elasticsearch output with compression)" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi