#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /usr/share/elma-tests/bin/elma_diag_imptcp_elasticsearch.sh
#

# async test for elasticsearch functionality

SCRIPT_DESCRIPTION="rsyslogd performance test with event forwarding to elasticsearch"
SCRIPT_AUTHOR="Joerg Heinemann"

elasticsearchdiagindex="diag"

# ######### Load ELMA configuration parameter, global functions and script header #########
#
source /etc/elma/elma.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $ELMA_LIB_DIR/elma-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
script_header ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

export testsdir="$ELMA_TESTS_LIB_DIR" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
cd $testsdir ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh init ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh delete-elasticsearch-index $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh startup-dq $RSYSLOG_CONF_DIR/elma_diag_imptcp_elasticsearch.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# send 1.000 messages of 200 bytes plus header, 100 times
# Only 9 option parameter will be accepted!

source $testsdir/diag.sh -d200 -m1000 -R100 -S0 -c1 -Ttcp -p13514 -t127.0.0.1 tcpflood ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh elasticsearch-index-check $elasticsearchdiagindex $TCPFLOOD_TOTALMESSAGES ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh elasticsearch-index-statistics $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh elasticsearch-index-status $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh elasticsearch-index-settings $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh elasticsearch-index-mappings $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# due to large messages, we need this time for the tcp receiver to settle...
#source $testsdir/diag.sh wait-tcp-receiver-to-settle 2 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# shut down rsyslogd when done processing messages
source $testsdir/diag.sh shutdown-when-empty ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# and wait for it to terminate
source $testsdir/diag.sh wait-shutdown ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

source $testsdir/diag.sh delete-elasticsearch-index $elasticsearchdiagindex ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $testsdir/diag.sh exit $RETVAL $testname ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
