#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /usr/share/elma-tests/bin/elma_nagios_probe.sh
#
# rsyslogd probe for Nagios 3.x
#
# Tested on openSUSE 13.1 with ELMA and rsyslog-testtools packages installed.
#

RETVAL=0
#IFS="_"
OPTIONS=$@
LOGGING=0

SCRIPT_DESCRIPTION="rsyslogd Nagios probe"
SCRIPT_AUTHOR="Joerg Heinemann"

elasticsearchdiagindex="nagios"
elasticsearchdiagindexnocompression=""

# ######### Load ELMA configuration parameter, global functions and script header #########
#
source /etc/elma/elma.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $ELMA_LIB_DIR/elma-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

export testsdir="$ELMA_TESTS_LIB_DIR" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
cd $testsdir ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# tcpflood options:
#
# -t 	target address (default 127.0.0.1)
# -p 	target port (default 13514)
# -c 	number of connections (default 1)
# -m 	number of messages to send (connection is random)
# -d 	amount of extra data to add to message. If present, the number itself will be added as third field, and the data bytes as forth. Add -r to randomize the amount of extra data included in the range 1..(value of -d).
# -r 	randomize amount of extra data added (-d must be > 0)
# -R 	number of times the test shall be run (very useful for gathering performance data and other repetitive things). Default: 1
# -S 	number of seconds to sleep between different runs (-R) Default: 30
# -T 	transport to use. Currently supported: "udp", "tcp" (default) Note: UDP supports a single target port, only
## -z 	private key file for TLS mode (not implemented into ELMA at this time)
## -Z 	cert (public key) file for TLS mode (not implemented into ELMA at this time)
## -L 	loglevel to use for GnuTLS troubleshooting (0-off to 10-all, 0 default) (not implemented into ELMA at this time)
#
# Example of option definitions: send 100 messages of 200 bytes plus header, 3 times, every second
#
# ELMA_PROBE_PARAMETER="-t127.0.0.1 -Tudp -p514 -d200 -m100 -R3 -S1 -P0"
#
# NOTE: Only 8 option parameter will be accepted!
#

source $testsdir/diag.sh delete-elasticsearch-index $elasticsearchdiagindex

source $testsdir/diag.sh $1 $2 $3 $4 $5 $6 $7 -P0 tcpflood
source $testsdir/diag.sh elasticsearch-index-check $elasticsearchdiagindex $TCPFLOOD_TOTALMESSAGES

source $testsdir/diag.sh delete-elasticsearch-index $elasticsearchdiagindex
source $testsdir/diag.sh exit $RETVAL $testname


if [ $RETVAL -eq "0" ]
then
	export TEST_SUMMARY="$TCPFLOOD_TOTALMESSAGES messages send (MPS: $MPS), received and properly indexed (Indexer Runtime: $INDEXERRUNTIME seconds)"
else
	export TEST_SUMMARY="See error report below"
fi

echo -e "$RETVAL|$TEST_SUMMARY|$PERF_DATA"
echo -e "$TEST_OUTPUT|"