#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /usr/share/elma-tests/lib/diag.sh
#

# this shell script provides commands to the common diag system. It enables
# test scripts to wait for certain conditions and initiate certain actions.
# needs support in config file.
# NOTE: this file should be included with "source diag.sh", as it otherwise is
# not always able to convey back states to the upper-level test driver

# begun 2009-05-27 by rgerhards
# This file is part of the rsyslog project, released under GPLv3

# ######### Load ELMA configuration parameter and global functions #########
#
source /etc/elma/elma.conf
source $ELMA_LIB_DIR/elma-global-functions.sh

#valgrind="valgrind --malloc-fill=ff --free-fill=fe --log-fd=1"
#valgrind="valgrind --tool=drd --log-fd=1"
#valgrind="valgrind --tool=helgrind --log-fd=1"
#valgrind="valgrind --tool=exp-ptrcheck --log-fd=1"

#set -o xtrace
#export RSYSLOG_DEBUG="debug nologfuncflow noprintmutexaction nostdout"
#export RSYSLOG_DEBUGLOG="log"



while getopts "d:m:R:S:c:T:p:t:P:" OPTION; do
     case $OPTION in
         d)
		# Message lenght (Byte)
		TCPFLOOD_d=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -d$TCPFLOOD_d"
             ;;
         r)
		# randomize amount of extra data added (-d must be > 0)
		TCPFLOOD_r=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -r$TCPFLOOD_r"
             ;;
         m)
		# Number of message to be sent during one tcpflood test
		TCPFLOOD_m=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -m$TCPFLOOD_m"
             ;;
         R)
		# Number of times tcpflood test has to be run
		TCPFLOOD_R=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -R$TCPFLOOD_R"
             ;;
         S)
		# Number of seconds to sleep between different tcpflood runs
		TCPFLOOD_S=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -S$TCPFLOOD_S"
             ;;
         c)
		# Number of concurrent tcpflood connections
		TCPFLOOD_c=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -c$TCPFLOOD_c"
             ;;
         T)
		# tcpflood transport protocol
		TCPFLOOD_T=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -T$TCPFLOOD_T"
             ;;
         p)
		# tcpflood rsyslog target port
		TCPFLOOD_p=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -p$TCPFLOOD_p"
             ;;
         t)
		# tcpflood rsyslog target address
		TCPFLOOD_t=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -t$TCPFLOOD_t"
             ;;
         P)
		# tcpflood PRI to be used for generated messages
		TCPFLOOD_P=$OPTARG
		TCPFLOOD_PARAMETER="$TCPFLOOD_PARAMETER -P$TCPFLOOD_P"
             ;;
         \?)
		echo "Invalid option: -$OPTARG"
		exit 1
		;;
         :)
		echo "Option -$OPTARG requires an argument."
		exit 1
		;;
     esac
done

shift $(($OPTIND -1))
OPTIND=""

case $1 in

   'init')	echo  --- `date +%T.%N | cut -c 1-12` - init -------------------------------------------------------------------
		echo

		# kill rsyslogd if it runs for some reason
   		$testsdir/diag.sh killrsyslog

		rm -f rsyslogd.started work-*.conf rsyslog.random.data
		rm -f rsyslogd2.started work-*.conf
		rm -f work rsyslog.out.log rsyslog2.out.log rsyslog.out.log.save # common work files
		rm -rf test-spool test-logdir stat-file1
		rm -f rsyslog.out.*.log work-presort rsyslog.pipe
		rm -f rsyslog.input rsyslog.empty
		rm -f core.* vgcore.*
		# Note: rsyslog.action.*.include must NOT be deleted, as it
		# is used to setup some parameters BEFORE calling init. This
		# happens in chained test scripts. Delete on exit is fine,
		# though.
		mkdir test-spool
		;;
   'exit')	echo
		echo  --- `date +%T.%N | cut -c 1-12` - exit -------------------------------------------------------------------
		echo
		rm -f rsyslogd.started work-*.conf elma_diag_common.conf
   		rm -f rsyslogd2.started diag-common2.conf rsyslog.action.*.include
		rm -f work rsyslog.out.log rsyslog2.out.log rsyslog.out.log.save # common work files
		rm -rf test-spool test-logdir stat-file1
		rm -f rsyslog.out.*.log rsyslog.random.data work-presort rsyslog.pipe
		rm -f rsyslog.input rsyslog.conf.tlscert stat-file1 rsyslog.empty

		if [ -z != $DATASIZE_COMPRESSED ] && [ -z != $DATASIZE ]; then
			COMPRESSIONRATIO=`echo "scale=2; 100 * ( ${DATASIZE} - ${DATASIZE_COMPRESSED} ) / ${DATASIZE}" | bc`
		fi

		echo  ===========================================================================================

		if [ "$2" -ne "0" ]; then
		  echo "FAIL: $3"
		else
		  echo "PASS: $3"
		  echo  -------------------------------------------------------------------------------------------
		  echo
		  echo "Raw message lenght (Byte):				$TCPFLOOD_d"
		  echo "Total message lenght (Byte):				$TOTALMESSAGELENGHT"
		  echo "Messages sent during one tcpflood test:			$TCPFLOOD_m"
		  echo "Number of tcpflood tests:				$TCPFLOOD_R"
		  echo "Seconds to sleep between tcpflood runs:			$TCPFLOOD_S"
		  echo "Concurrent tcpflood connections:			$TCPFLOOD_c"
		  echo "tcpflood transport protocol:				$TCPFLOOD_T"
		  echo "tcpflood rsyslog target port:				$TCPFLOOD_p"
		  echo "tcpflood rsyslog target address:			$TCPFLOOD_t"
		  echo
		  echo "Total messages:						$TCPFLOOD_TOTALMESSAGES"
		  echo "Total tcpflood runtime (milli seconds):			$TCPFLOOD_RUNTIME"
		  echo "Loging rate (MPS):					$MPS"
		  echo "Transmission speed (MBit/s):				$TRANSMISSIONSPEED"
		  echo
		  echo "Compressed data size (MByte):				$DATASIZE_COMPRESSED_MB"
		  echo "Compressed data indexing runtime (milli seconds):	$COMPRESSED_INDEXING_RUNTIME"
		  echo "Compression method:					$COMPRESSIONMETHOD"
		  echo "Compression ratio (%):					$COMPRESSIONRATIO"
		  echo "Decompression runtime (milli seconds):			$DECOMPRESSION_RUNTIME"
		  echo
		  echo "Uncompressed data size (MByte):				$DATASIZE_MB"
		  echo "Uncompressed data indexing runtime (milli seconds):	$INDEXING_RUNTIME"
		  echo
		fi
		echo  ===========================================================================================
		echo
		;;
   'startup') # start rsyslogd with default params. $2 is the config file name to use
   		# returns only after successful startup, $3 is the instance (blank or 2!)
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - start rsyslogd ---------------------------------------------------------
		echo
		COMPRESSIONMETHOD=`grep "OMFileZipLevel" $2`
		cat $2
		echo
		echo
		echo
		cat $RSYSLOG_CONF_DIR/elma_diag_common.conf
		echo
              $valgrind $SBIN_DIR/rsyslogd -u2 -n -irsyslog$3.pid -M/usr/lib64/rsyslog -f$2 &
   		$testsdir/diag.sh wait-startup $3
		;;
   'startup-dq') # start rsyslogd with default and disk queue params. $2 is the config file name to use
   		# returns only after successful startup, $3 is the instance (blank or 2!)
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - start rsyslogd with disk queue params ----------------------------------
		echo
		COMPRESSIONMETHOD=`grep "OMFileZipLevel" $2`
		cat $2
		echo
		echo
		echo
		cat $RSYSLOG_CONF_DIR/elma_diag_common.conf
		echo
              $valgrind $SBIN_DIR/rsyslogd -u2 -n -irsyslog$3.pid -M/usr/lib64/rsyslog -f$2 &
   		$testsdir/diag.sh wait-startup $3
		;;
   'startup-vg') # start rsyslogd with default params under valgrind control. $2 is the config file name to use
   		# returns only after successful startup, $3 is the instance (blank or 2!)
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - start rsyslogd under valgrind control ----------------------------------
		echo
		COMPRESSIONMETHOD=`grep "OMFileZipLevel" $2`
		cat $2
		echo
		echo
		echo
		cat $RSYSLOG_CONF_DIR/elma_diag_common.conf
		echo
		echo
		echo
              valgrind --log-fd=1 --error-exitcode=10 --malloc-fill=ff --free-fill=fe --leak-check=full $SBIN_DIR/rsyslogd -u2 -n -irsyslog$3.pid -M/usr/lib64/rsyslog -f$2 &
		sleep 1
   		$testsdir/diag.sh wait-startup $3
		;;
   'wait-startup') # wait for rsyslogd startup ($2 is the instance)
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - wait for rsyslogd startup ----------------------------------------------
		echo
		while test ! -f rsyslog$2.pid; do
			msleep 100 # wait 100 milliseconds
		done
		while test ! -f rsyslogd$2.started; do
			msleep 100 # wait 100 milliseconds
		done
		echo "rsyslogd$2 started with pid " `cat rsyslog$2.pid`
		;;
   'wait-shutdown')  # actually, we wait for rsyslog.pid to be deleted. $2 is the
   		# instance
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - wait for rsyslogd shutdown ---------------------------------------------
		echo
		while test -f rsyslog$2.pid; do
			msleep 100 # wait 100 milliseconds
		done
		if [ -e core.* ]
		then
		   echo "ABORT! core file exists, starting interactive shell"
		   bash
		   exit 1
		fi
		;;
   'wait-shutdown-vg')  # actually, we wait for rsyslog.pid to be deleted. $2 is the
   		# instance
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - wait for rsyslogd-vg shutdown ------------------------------------------
		echo
		wait `cat rsyslog.pid`
		export RSYSLOGD_EXIT=$?
		echo
		echo rsyslogd run exited with $RSYSLOGD_EXIT
		if [ -e core.* ]
		then
		   echo "ABORT! core file exists, starting interactive shell"
		   bash
		   exit 1
		fi
		;;
   'killrsyslog') # check if rsyslog instance exists and, if so, kill it
		if [ -e "rsyslog.pid" ]
		then
		  echo rsyslog.pid exists, trying to shut down rsyslogd process `cat rsyslog.pid`.
		  kill -9 `cat rsyslog.pid`
		  sleep 1
		fi
		if [ -e "rsyslog2.pid" ]
		then
		  echo rsyslog2.pid exists, trying to shut down rsyslogd process `cat rsyslog2.pid`.
		  kill -9 `cat rsyslog2.pid`
		  sleep 1
		fi
		;;
   'check-exit-vg') # wait for main message queue to be empty. $2 is the instance.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - check exit vg ----------------------------------------------------------
		echo
		if [ "$RSYSLOGD_EXIT" -eq "10" ]
		then
			echo "valgrind run FAILED with exceptions - terminating"
			exit 1
		fi
		;;
   'get-mainqueuesize') # show the current main queue size
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - get the current main queue size ----------------------------------------
		echo
		if [ "$2" == "2" ]
		then
			echo getmainmsgqueuesize | diagtalker -p13501
		else
			echo getmainmsgqueuesize | diagtalker
		fi
		;;
   'wait-queueempty') # wait for main message queue to be empty. $2 is the instance.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - wait for main message queue to be empty --------------------------------
		echo
		if [ "$2" == "2" ]
		then
			echo WaitMainQueueEmpty | diagtalker -p13501
		else
			echo WaitMainQueueEmpty | diagtalker
		fi
		;;
   'wait-tcp-receiver-to-settle') # due to large messages, we need this time for the rsyslogd tcp receiver to settle.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - time for the rsyslogd tcp receiver to settle ---------------------------
		echo
		sleep $2
		;;
   'shutdown-when-empty') # shutdown rsyslogd when main queue is empty. $2 is the instance.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - shutdown rsyslogd when main queue is empty -----------------------------
		echo
		if [ "$2" == "2" ]
		then
		   echo Shutting down instance 2
		fi
   		$testsdir/diag.sh wait-queueempty $2
		kill `cat rsyslog$2.pid`
		# note: we do not wait for the actual termination!
		;;
   'shutdown-immediate') # shutdown rsyslogd immediate without emptying the queue. $2 is the instance.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - shutdown rsyslogd immediate without emptying the queue -----------------
		echo
		kill `cat $testsdir/rsyslog.pid`
		# note: we do not wait for the actual termination!
		;;
   'delete-elasticsearch-index') # delete elasticsearch diag index.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - delete elasticsearch diag index ----------------------------------------
		echo
		curl $ES_AUTH -XDELETE $ES_URL/$2/events/; export RETVAL=$?
		echo
		;;
   'elasticsearch-index-check') # count elasticsearch indexed messages.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - count elasticsearch indexed messages -----------------------------------
		echo

		# waiting time in milliseconds
		SLEEPINGTIME="1000"

		# Abort after milliseconds (2 min)
		ABORTRUNTIME="120000"

		RECEIVEDMESSAGES="0"
		RECEIVEDMESSAGESPREVIOUS="0"

		while [ $RECEIVEDMESSAGES -lt $3 ]
		do
			msleep $SLEEPINGTIME
			INDEXERRUNTIME=$(( $INDEXERRUNTIME + $SLEEPINGTIME ))
			RECEIVEDMESSAGES=$(curl $ES_AUTH -s -XGET $ES_URL/$2/_count | awk -F':' '{ print $2 }' | awk -F',' '{ print $1 }' )
			echo "$RECEIVEDMESSAGES messages indexed ($INDEXERRUNTIME milliseconds - Abort in $ABORTRUNTIME milliseconds)"
			if [ $RECEIVEDMESSAGES -eq $RECEIVEDMESSAGESPREVIOUS ]
			then
				ABORTRUNTIME=$(( $ABORTRUNTIME - $SLEEPINGTIME ))
				if [ "$ABORTRUNTIME" == "0" ]
				then
					echo
					echo  ===========================================================================================
		   			echo "ABORT! giving up - check rsyslog or elasticsearch configuration"
					echo  ===========================================================================================
					echo
					export RETVAL="1"
		  			LOCAL_TEST_OUTPUT="Sequence error detected!!!"
		  			LOCAL_PERF_DATA=""
				fi
			else
				RECEIVEDMESSAGESPREVIOUS=$RECEIVEDMESSAGES
#				ABORTRUNTIME="180000"
				export RETVAL="0"
				export INDEXERRUNTIME=$(( $INDEXERRUNTIME / 1000 ))
		  		LOCAL_TEST_OUTPUT="OK - Sequence check pass in $INDEXERRUNTIME seconds"
		  		LOCAL_PERF_DATA="$INDEXERRUNTIME"
			fi
		done
		export TEST_OUTPUT="$TEST_OUTPUT\n$LOCAL_TEST_OUTPUT"
		export PERF_DATA="$PERF_DATA$LOCAL_PERF_DATA"
		echo
		echo "$LOCAL_TEST_OUTPUT - $LOCAL_PERF_DATA"
		;;
   'elasticsearch-index-statistics') # show elasticsearch index statistics.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - show elasticsearch index statistics ------------------------------------
		echo
		curl $ES_AUTH -XGET $ES_URL/$2/_stats?pretty=true; export RETVAL=$?
		echo
		;;
   'elasticsearch-index-status') # show elasticsearch index status.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - show elasticsearch index status ----------------------------------------
		echo
		curl $ES_AUTH -XGET $ES_URL/$2/_status?pretty=true; export RETVAL=$?
		echo
		;;
   'elasticsearch-index-settings') # show elasticsearch index settings.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - show elasticsearch index settings --------------------------------------
		echo
		curl $ES_AUTH -s -XGET $ES_URL/$2/_settings?pretty=true; export RETVAL=$?
		echo

		COMPRESSIONMETHOD=$(curl $ES_AUTH -s -XGET $ES_URL/$2/_settings?pretty=true | grep "store.compress" | sed -e 's/[ ",]*//g ; s/:/ = /')

		if [ "$COMPRESSIONMETHOD" == "index.store.compress.stored = true" ]
		then
			DATASIZE_COMPRESSED=$(curl $ES_AUTH -s -XGET $ES_URL/$2/_stats?pretty=true | grep -m 1 "size_in_bytes" | awk -F',' '{ print $1 }' | awk -F' ' '{ print $3 }' )
			DATASIZE_COMPRESSED_MB=`echo "scale=2; ${DATASIZE_COMPRESSED} / 1024 / 1024" | bc`
			COMPRESSED_INDEXING_RUNTIME=$(curl $ES_AUTH -s -XGET $ES_URL/$2/_stats?pretty=true | grep -m 1 "index_time_in_millis" | awk -F',' '{ print $1 }' | awk -F' ' '{ print $3 }' )
		else
			DATASIZE=$(curl $ES_AUTH -s -XGET $ES_URL/$2/_stats?pretty=true | grep -m 1 "size_in_bytes" | awk -F',' '{ print $1 }' | awk -F' ' '{ print $3 }' )
			DATASIZE_MB=`echo "scale=2; ${DATASIZE} / 1024 / 1024" | bc`
			INDEXING_RUNTIME=$(curl $ES_AUTH -s -XGET $ES_URL/$2/_stats?pretty=true | grep -m 1 "index_time_in_millis" | awk -F',' '{ print $1 }' | awk -F' ' '{ print $3 }' )
		fi
		;;
   'elasticsearch-index-mappings') # show elasticsearch index mappings.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - show elasticsearch index mappings --------------------------------------
		echo
		curl $ES_AUTH -s -XGET $ES_URL/$2/_mapping?pretty=true; export RETVAL=$?
		echo
		;;
   'tcpflood') # check tcpflood options and run tcpflood
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - check tcpflood options and run tcpflood --------------------------------
		echo

		TCPFLOOD="tcpflood -X -n1$TCPFLOOD_PARAMETER"

		TCPFLOOD_START=$(date +%s%N)
		echo "$TCPFLOOD"
		echo
		$TCPFLOOD; export RETVAL=$?
		if [ "$?" -ne "0" ]; then
		  echo "error during tcpflood! see rsyslog.out.log.save for what was written"
		  cp rsyslog.out.log rsyslog.out.log.save
		  exit 1
		fi
		TCPFLOOD_END=$(date +%s%N)

		# tcpflood default values
		#
		if [ -z $TCPFLOOD_d ]; then
			TCPFLOOD_d=""
		fi
		if [ -z $TCPFLOOD_m ]; then
			TCPFLOOD_m="0"
		fi
		if [ -z $TCPFLOOD_R ]; then
			TCPFLOOD_R="1"
		fi
		if [ -z $TCPFLOOD_S ]; then
			TCPFLOOD_S="30"
		fi
		if [ -z $TCPFLOOD_c ]; then
			TCPFLOOD_c="1"
		fi
		if [ -z $TCPFLOOD_T ]; then
			TCPFLOOD_T="tcp"
		fi
		if [ -z $TCPFLOOD_p ]; then
			TCPFLOOD_p="13514"
		fi
		if [ -z $TCPFLOOD_t ]; then
			TCPFLOOD_t="127.0.0.1"
		fi
		if [ -z $TCPFLOOD_P ]; then
			TCPFLOOD_P="167"
		fi

		# calculation of total messages
		export TCPFLOOD_TOTALMESSAGES=$(( $TCPFLOOD_m * $TCPFLOOD_R ))
		# runtime in ms
		TCPFLOOD_RUNTIME=$((( $TCPFLOOD_END - $TCPFLOOD_START ) / 1000000 ))
		# calculation of MPS
		export MPS=$(( $TCPFLOOD_TOTALMESSAGES * 1000 / $TCPFLOOD_RUNTIME ))

		echo
		echo "Raw message lenght (Byte):			$TCPFLOOD_d"
		echo "Messages sent during one tcpflood test:		$TCPFLOOD_m"
		echo "Number of tcpflood tests:			$TCPFLOOD_R"
		echo "Seconds to sleep between tcpflood runs:		$TCPFLOOD_S"
		echo "Concurrent tcpflood connections:		$TCPFLOOD_c"
		echo "tcpflood transport protocol:			$TCPFLOOD_T"
		echo "tcpflood rsyslog target port:			$TCPFLOOD_p"
		echo "tcpflood rsyslog target address:		$TCPFLOOD_t"
		echo
		echo "Total messages:					$TCPFLOOD_TOTALMESSAGES"
		echo "Total tcpflood runtime (milli seconds):		$TCPFLOOD_RUNTIME"
		echo "Loging rate (MPS):				$MPS"

		LOCAL_TEST_OUTPUT="OK - tcpflood check pass in $TCPFLOOD_RUNTIME milli seconds ($TCPFLOOD)"
		LOCAL_PERF_DATA="$TCPFLOOD_TOTALMESSAGES;$(( $TCPFLOOD_RUNTIME / 1000 ));"

		export TEST_OUTPUT="$LOCAL_TEST_OUTPUT"
		export PERF_DATA="$LOCAL_PERF_DATA"
		echo
		echo "$LOCAL_TEST_OUTPUT - $LOCAL_PERF_DATA"
		;;
   'injectmsg') # inject messages via our inject interface (imdiag)
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - inject messages via our inject interface imdiag ------------------------
		echo
		echo injecting $3 messages
		echo injectmsg $2 $3 $4 $5 | diagtalker
		# TODO: some return state checking? (does it really make sense here?)
		;;
   'check-mainq-spool') # check if mainqueue spool files exist, if not abort (we just check .qi).
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - check if mainqueue spool files exist -----------------------------------
		echo There must exist some files now:
		ls -l test-spool
		if test ! -f test-spool/mainq.qi; then
		  echo "error: mainq.qi does not exist where expected to do so!"
		  ls -l test-spool
		  exit 1
		fi
		;;
   'seq-check') # do the usual sequence check to see if everything was properly received. $2 is the instance.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - sequence check to see if everything was properly received --------------
		echo
		rm -f work
		cp rsyslog.out.log work-presort
		sort < rsyslog.out.log > work

		sed -i -e '/\/\/www.rsyslog.com"]/d' work

		# $4... are just to have the abilit to pass in more options...
		# add -v to chkseq if you need more verbose output
		chkseq -fwork -s$2 -e$3 $4 $5 $6 $7; export RETVAL=$?

		if [ "$RETVAL" -ne "0" ]; then
		  LOCAL_TEST_OUTPUT="Sequence error detected!!!"
		  LOCAL_PERF_DATA=""
		else
		  LOCAL_TEST_OUTPUT="OK - Sequence check pass"
		  LOCAL_PERF_DATA=""
		fi

		rm -f work

		export TEST_OUTPUT="$TEST_OUTPUT\n$LOCAL_TEST_OUTPUT"
		export PERF_DATA="$PERF_DATA\n$LOCAL_PERF_DATA"
		echo "$LOCAL_TEST_OUTPUT - $LOCAL_PERF_DATA"
		;;
   'seq-check2') # do the usual sequence check to see if everything was properly received. This is
   		# a duplicateof seq-check, but we could not change its calling conventions without
		# breaking a lot of exitings test cases, so we preferred to duplicate the code here.
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - sequence check2 to see if everything was properly received -------------
		echo
		rm -f work2
		sort < rsyslog2.out.log > work2

		sed -i -e '/\/\/www.rsyslog.com"]/d' work2

		# $4... are just to have the abilit to pass in more options...
		# add -v to chkseq if you need more verbose output
		chkseq -fwork2 -s$2 -e$3 $4 $5 $6 $7; export RETVAL=$?

		if [ "$RETVAL" -ne "0" ]; then
		  LOCAL_TEST_OUTPUT="Sequence error detected!!!"
		  LOCAL_PERF_DATA=""
		else
		  LOCAL_TEST_OUTPUT="OK - Sequence check pass"
		  LOCAL_PERF_DATA=""
		fi

		rm -f work2

		export TEST_OUTPUT="$TEST_OUTPUT\n$LOCAL_TEST_OUTPUT"
		export PERF_DATA="$PERF_DATA\n$LOCAL_PERF_DATA"
		echo "$LOCAL_TEST_OUTPUT - $LOCAL_PERF_DATA"
		;;
   'gzip-seq-check') # do the usual sequence check, but for gzip files
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - sequence check gzip to see if everything was properly received ---------
		echo
		rm -f work
		ls -l rsyslog.out.log

		DATASIZE_COMPRESSED=`stat -c %s rsyslog.out.log`
		DATASIZE_COMPRESSED_MB=`echo "scale=2; ${DATASIZE_COMPRESSED} / 1024 / 1024" | bc`

		DECOMPRESSION_START=$(date +%s%N)
		gunzip < rsyslog.out.log | sort > work
		if [ "$?" -ne "0" ]; then
		  echo "error during decompression! see rsyslog.out.log.save for what was written"
		  cp rsyslog.out.log rsyslog.out.log.save
#		  exit 1
		fi
		DECOMPRESSION_END=$(date +%s%N)

		# runtime in ms
		DECOMPRESSION_RUNTIME=$((( $DECOMPRESSION_END - $DECOMPRESSION_START ) / 1000000 ))

		sed -i -e '/\/\/www.rsyslog.com"]/d' work

		ls -l work
		echo

		DATASIZE=`stat -c %s work`
		DATASIZE_MB=`echo "scale=2; ${DATASIZE} / 1024 / 1024" | bc`

		# calculation of total messages lengh
		TOTALMESSAGELENGHT=$(( $DATASIZE / $TCPFLOOD_TOTALMESSAGES ))

		TRANSMISSIONSPEED=`echo "scale=2; ( ${DATASIZE} * 8 / ${TCPFLOOD_RUNTIME} * 1000 / 1024 / 1024 )" | bc`

		head -1 work
		echo

		# $4... are just to have the abilit to pass in more options...
		chkseq -fwork -v -s$2 -e$3 $4 $5 $6 $7; export RETVAL=$?

		if [ "$RETVAL" -ne "0" ]; then
		  LOCAL_TEST_OUTPUT="Sequence error detected!!!"
		  LOCAL_PERF_DATA=""
		else
		  LOCAL_TEST_OUTPUT="OK - Sequence check pass"
		  LOCAL_PERF_DATA=""
		fi

		rm -f work

		export TEST_OUTPUT="$TEST_OUTPUT\n$LOCAL_TEST_OUTPUT"
		export PERF_DATA="$PERF_DATA\n$LOCAL_PERF_DATA"
		echo "$LOCAL_TEST_OUTPUT - $LOCAL_PERF_DATA"
		;;
   'nettester') # perform nettester-based tests
   		# use -v for verbose output!
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - perform nettester-based tests ------------------------------------------
		echo
		nettester -t$2 -i$3 $4
		if [ "$?" -ne "0" ]; then
		  exit 1
		fi
		;;
   'setzcat')   # find out name of zcat tool
		echo
		echo  --- `date +%T.%N | cut -c 1-12` - find out name of zcat tool ---------------------------------------------
		echo
		if [ `uname` == SunOS ]; then
		   ZCAT=gzcat
		else
		   ZCAT=zcat
		fi
		;;
   *)
		echo "Invalid argument: $1"
		;;
esac